"""
Create by yy on 2019/9/23
"""
INSOMNIA_MUSIC_DATABASE_CONFIG = {
    "MYSQL_DATABASE": "insomnia_music",
    "MYSQL_USERNAME": "yy",
    "MYSQL_PASSWORD": "wyysdsa!",
    "MYSQL_HOST": "127.0.0.1",
    "MYSQL_PORT": "3306"
}